# encoding=utf8
'''
用于微服务的框架，没有认证的url统一返回401
'''
import json
from functools import wraps
from flask import request, session,  g,  make_response
from mwauth.base_auth import BaseAuth

class SessionAuth(BaseAuth):
    """docstring for ClassName"""
    app = None
    db = None
    user_class = None

    def __init__(self, app=None):
        super(SessionAuth, self).__init__()
        #if not self.home_url:
            # 用于登录用户进入登录页面跳转链接问题
         #   self.home_url = "/"
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app
        app.auth = self

    def valid_login(self, func):
        @wraps(func)
        def _request_login(*args, **kw):
            user_id = session.get('uid')
            if user_id:
                # 如果没有manageuser key，则补充到session中，确保管理者权限设定页面有效
                if not session.get('manageuser') and hasattr(self.user_class,'manageuser'):
                    userObj = self.user_class.FindUser(session.get('uname'))
                    session.update(userObj.to_json())
                else:
                    userObj = self.user_class()
                user_dict = userObj.to_json()
                for key in user_dict:
                    setattr(userObj, key, session.get(key))
                g.current_user = userObj
                return func(*args, **kw)
            # 没有认证则返回401
            response = make_response()
            response.status_code = 401
            return response
        return _request_login





#encoding=utf8
from collections import namedtuple
User = namedtuple('User', ['uid', 'uname', 'systemuser','manageuser','manageuserid'])

class BaseAuth(object):
    """docstring for ClassName"""
    #
    def init_app(self, app):
        pass

    def valid_login(self, func):
        pass



# encoding=utf8
'''
在没有认证的情况下，直接跳转到登录页面
'''
import json
import urllib

# import datetime
# import hashlib
from functools import wraps
from flask import request, session, redirect, g, render_template, \
    Blueprint, make_response, current_app
from mwauth.base_auth import BaseAuth
from werkzeug.wrappers import Response
from werkzeug.wsgi import get_current_url
from werkzeug.urls import uri_to_iri, url_quote, url_join, iri_to_uri

auth = Blueprint("auth", __name__, static_folder="./static", template_folder="./templates")


def get_host(environ):
    if 'HTTP_HOST' in environ:
        rv = environ['HTTP_HOST']
    else:
        rv = environ['SERVER_NAME']
        if (environ['wsgi.url_scheme'], environ['SERVER_PORT']) not \
                in (('https', '443'), ('http', '80')):
            rv += ':' + environ['SERVER_PORT']
    return rv


def get_current_url(environ):
    tmp = [environ['wsgi.url_scheme'], '://', get_host(environ)]
    cat = tmp.append
    return uri_to_iri(''.join(tmp) + '/')
    cat(url_quote(wsgi_get_bytes(environ.get('SCRIPT_NAME', ''))).rstrip('/'))
    cat('/')
    return uri_to_iri(''.join(tmp))


def correct_location(location, request):
    '''
    把location 改为完整的URL
    :param location: url path 如：/auth/login
    :param request:
    :return: 完整url，如:http://location:8082/auth/login
    '''
    current_url = get_current_url(request.environ)
    current_url = iri_to_uri(current_url)
    return url_join(current_url, location)


class Redirect_Respose(Response):
    # pass
    def __init__(self, response=None, status=None, headers=None,
                 mimetype=None, content_type=None, direct_passthrough=False):
        super().__init__(response, status, headers,
                         mimetype, content_type, direct_passthrough)
        # 屏蔽 wrappers.py 的 1143行代码，不对location做host纠正
        self.autocorrect_location_header = False


class SessionAuth(BaseAuth):
    """docstring for ClassName"""
    app = None
    db = None
    user_class = None
    home_url = None

    def __init__(self, app=None, home_url=None):
        super(SessionAuth, self).__init__()
        self.home_url = home_url
        if not self.home_url:
            # 用于登录用户进入登录页面跳转链接问题
            self.home_url = "/"
        if app is not None:
            self.init_app(app)

    def init_app(self, app, home_url=None):
        self.app = app
        app.auth = self
        if home_url:
            self.home_url = home_url + '/' if not home_url.endswith('/') else home_url
        self.app.register_blueprint(auth, url_prefix='%sauth' % self.home_url)

    def valid_login(self, func):
        @wraps(func)
        def _request_login(*args, **kw):
            url_to = request.args.get("to")
            user_id = session.get('uid')
            if user_id:
                userObj = self.user_class()
                user_dict = userObj.to_json()
                for key in user_dict:
                    setattr(userObj, key, session.get(key))
                g.current_user = userObj
                # g.redirect_url = request.headers.get("Referer","/")
                return func(*args, **kw)
            # 判断是否为ajax请求
            request_with = request.headers.get("x-requested-with")
            if request_with and request_with.lower() == "XMLHttpRequest".lower():
                response = make_response()
                response.status_code = 401
                return response
            if not url_to:
                if request.full_path[-1] == '?':
                    full_path = request.full_path[:-1]
                else:
                    full_path = request.full_path
                # 如果没有to 参数，则转到根
                if full_path == '/':
                    url_to = '../'
                else:
                    url_to = urllib.parse.quote(full_path)
            return redirect(correct_location("%sauth/login?to=%s" % (self.home_url, url_to), request),
                            Response=Redirect_Respose)

        return _request_login

    def valid_login_redirect(self, redirect_url=None):
        def _valid_login(func):
            r_to_url = redirect_url

            @wraps(func)
            def _request_login(*args, **kw):
                url_to = request.args.get("to")
                user_id = session.get('uid')
                if user_id:
                    userObj = self.user_class()
                    user_dict = userObj.to_json()
                    for key in user_dict:
                        setattr(userObj, key, session.get(key))
                    g.current_user = userObj
                    # g.redirect_url = request.headers.get("Referer","/")
                    return func(*args, **kw)
                # 判断是否为ajax请求
                request_with = request.headers.get("x-requested-with")
                if request_with and request_with.lower() == "XMLHttpRequest".lower():
                    response = make_response()
                    response.status_code = 401
                    return response
                if not url_to:
                    if request.full_path[-1] == '?':
                        full_path = request.full_path[:-1]
                    else:
                        full_path = request.full_path
                    # 如果没有to 参数，则转到根
                    if full_path == '/':
                        url_to = '../'
                    else:
                        url_to = urllib.parse.quote(full_path)
                if r_to_url is not None:
                    return redirect(correct_location(redirect_url, request),
                                    Response=Redirect_Respose)
                else:
                    return redirect(correct_location("%sauth/login?to=%s" % (self.home_url, url_to), request),
                                    Response=Redirect_Respose)

            return _request_login

        return _valid_login


def valid_cookie():
    cookies = request.cookies
    if not cookies: return False
    sid = cookies.get("sessionid")
    if sid and sid == session.get("uid"): return True
    return False


@auth.route("/login", methods=["GET", "POST"])
def login():
    if request.method == "GET":
        url = request.args.get("to", current_app.auth.home_url)
        user_id = session.get("uid")
        # if not user_id:
        #     user_id = session.get(b'uid')
        if user_id:
            return redirect(correct_location(urllib.parse.unquote(url), request), Response=Redirect_Respose)
        return render_template("auth/login.html", url=urllib.parse.unquote(url),base_uri=current_app.auth.home_url)
    username = request.form.get("username")
    pwd = request.form.get("password")
    # 请指定user_class类
    assert current_app.auth.user_class != None
    userObj = current_app.auth.user_class.FindUser(username)

    if not userObj: return json.dumps({"msg": "用户名不存在"})
    if not userObj.check(pwd): return json.dumps({"msg": "密码错误"})
    userjson = userObj.to_json()

    session.update(userjson)
    response = make_response(json.dumps({"success": True}))
    return response


@auth.route("/logout", methods=["GET", "POST"])
def logout():
    session.clear()
    return redirect(correct_location("%sauth/login"%current_app.auth.home_url, request), Response=Redirect_Respose)

@auth.route("/user-logout", methods=["GET", "POST"])
def user_logout():
    session.clear()
    response = make_response(json.dumps({"success": True}))
    return response



#encoding=utf8

import json
from functools import wraps
from flask import request,  g, \
    Blueprint
from mwauth.base_auth import BaseAuth
from werkzeug.wrappers import Response
from redis import Redis

# key的过期时间，默认为7天
expire_time  = 7*24*60*60


auth = Blueprint("auth", __name__)

def not_authenticated():
    """Sends a 401 response that enables basic auth"""
    return Response('Could not verify your access level for that URL.\n'
                    'You have to login with proper credentials', 401,
                    {'WWW-Authenticate': 'Basic realm="Login Required"'})

class BasicAuth(BaseAuth):
    """docstring for ClassName"""
    app = None
    user_class = None
    redis = None
    def __init__(self,app=None,redis=None):
        super(BasicAuth, self).__init__()
        if app is not None:
            self.init_app(app,redis)

    def get_rds_key(self):
        return 'session:%s'%request.headers.environ['HTTP_AUTHORIZATION']

    def get_user_cache(self):
        userinfo = self.redis.get(self.get_rds_key())
        if userinfo is not None:
            try:
                userinfo = json.loads(userinfo.decode())
                user = self.user_class()
                for k, v in userinfo.items():
                    setattr(user, k, v)
            except:#如果userinfo的格式错误，不能创建物件就成资料库中载入资料
                self.redis.delete(self.get_rds_key())
                return None
            return user
        return None

    def check_auth(self,username, password):
        """This function is called to check if a username /
        password combination is valid.
        """
        user = self.get_user_cache()
        if user is not None:
            g.current_user = user
            # 重置key的有效期为2天
            self.redis.expire(self.get_rds_key(),expire_time)
            return True
        # 请指定user_class类
        assert self.user_class != None
        try:
            user = self.user_class.FindUser(username)
        except Exception as e:
            self.app.logger.error('FindUser error:'+str(e))
            user = None
        if user and user.check(password):
            g.current_user = user
            self.redis.setex(self.get_rds_key(),json.dumps(user.to_json()),expire_time)
            return True
        return False

    def init_app(self, app,redis):
        self.app = app
        app.auth = self
        assert redis != None
        self.redis =redis
        assert self.redis.provider_class == Redis,"请使用提示格式创建rds:rds = FlaskRedis(app,strict=False)"
        self.app.register_blueprint(auth)

    def valid_login(self, func):
        @wraps(func)
        def _request_login(*args, **kwargs):
            auth = request.authorization
            if not auth or not self.check_auth(auth.username, auth.password):
                return not_authenticated()
            return func(*args, **kwargs)
        return _request_login






from mwauth.base_auth import BaseAuth,User
from flask import request,make_response,g,current_app,session
from functools import wraps
from .redis_session import RedisSessionInterface
class KongAuth(BaseAuth):
    # '''
    # 1，在request前提取 user info
    # '''
    def __set_auth(self):
        '''
        保存用户，jwt等auth info
        :return:
        '''
        if not isinstance(current_app.session_interface,RedisSessionInterface):
            raise Exception('请在app/__init__.py的create_app_swagger方法中增加下列代码：\n app.session_interface = RedisSessionInterface(app, rds)')
        if current_app.config.get('DEVELOPMENT', False):
            g.user_name = current_app.config.get('LOGIN_USER_NAME')  # 'user_dev'
            g.user_id = current_app.config.get('LOGIN_USER_ID')
            g.current_user = User(uid=current_app.config.get('LOGIN_USER_ID'),
                                  uname=current_app.config.get('LOGIN_USER_NAME'),
                                  systemuser=current_app.config.get('LOGIN_USER_SYSTEMUSER', False),
                                  manageuser=current_app.config.get('LOGIN_USER_MANAGEUSER', False),
                                  manageuserid=current_app.config.get('LOGIN_USER_MANAGEUSER_ID'))
            g.jwt = session.sid
            session.update({'uid':g.current_user.uid,'uname':g.current_user.uname,
                            'systemuser':g.current_user.systemuser,
                            'manageuser':g.current_user.manageuser,
                            'manageuserid':g.current_user.manageuserid})
            return
        # 通过了jwt 认证的 api 一定会回传username和userid，否则视为kong的认证不成功
        g.user_name = request.headers.get('X-Consumer-Username')
        g.user_id = request.headers.get('X-Consumer-Custom-Id')
        g.jwt = request.args.get('jwt')
        if g.jwt is None:
            # jwt header authorization: bearer jwt...
            jwt = request.headers.get('authorization', None)
            if jwt:
                # 去掉前面的bearer
                g.jwt = jwt[7:]
            elif session:
                g.jwt = session.sid
                # session 认证可能不经过kong，但也需保证auth 成功，确保代码兼容
                g.user_name = session.get('uname')
                g.user_id = session.get('uid')
            else:
                assert False,'header，query，和cookie中都没有 jwt？'
        if not session:
            # 当非session认证时，还需保存session，保证app中的代码的兼容性
            session.sid = g.jwt
            # 临时session 不需要送cookie，避免浪费流量
            session.seend_cookie = False
            mysession = current_app.session_interface.get_session(g.jwt)
            session.update(dict(mysession))
        g.current_user = User(uid=session.get('uid'), uname=session.get('uname'),
                              systemuser=session.get('systemuser', False),
                              manageuser=session.get('manageuser', False),
                              manageuserid=session.get('manageuserid'))


    def __init__(self, app=None):
        super(KongAuth, self).__init__()
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app
        app.auth = self
        # self.app.register_blueprint(auth)

    def valid_login(self, func):
        @wraps(func)
        def _request_login(*args, **kw):
            # 给g 赋 用户等信息
            self.__set_auth()
            if g.user_name or g.user_id :
                return func(*args, **kw)
            # 没有认证则返回401
            response = make_response()
            response.status_code = 401
            return response
        return _request_login

# @auth.before_app_request
# def before_app_request():
#     if current_app.config.get('DEVELOPMENT',False):
#         g.user_name = current_app.config.get('LOGIN_USER_NAME')#'user_dev'
#         g.user_id = current_app.config.get('LOGIN_USER_ID')
#         g.jwt = ''
#         return
#     else:
#         g.user_name = request.headers.get('X-Consumer-Username')
#         g.user_id = request.headers.get('X-Consumer-Custom-Id')
#     g.jwt = request.args.get('jwt')
#     if g.jwt is None:
#         # jwt header authorization: bearer jwt...
#         jwt = request.headers.get('authorization', None)
#         if jwt:
#             # 去掉前面的bearer
#             g.jwt = jwt[7:]






from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'readme.md'), encoding='utf-8') as f:
    long_description = f.read()

# Arguments marked as "Required" below must be included for upload to PyPI.
# Fields marked as "Optional" may be commented out.

setup(
    # $ pip install mwauth
    # And where it will live on PyPI: https://pypi.org/project/mwauth/
    name='mwauth',  # Required
    version='0.4.17',  # Required
    description='maxwin auth',  # Required
    long_description=long_description,  # Optional
    long_description_content_type='text/markdown',  # Optional (see note above)
    url='https://bitbucket.org/maxwin-inc/auth/src/master/',  # Optional
    author='cxhjet',  # Optional
    author_email='cxhjet@163.com',  # Optional
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',
        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='maxwin commonlib auth',  # Optional
    packages=find_packages(exclude=['test','__pycache__']),  # Required
    install_requires=['flask','flask_redis'],  # Optional
    include_package_data=True,
    # package_data={  # Optional
    #     '': ['*.*'],
    # },

    # data_files=[('my_data', ['data/data_file'])],  # Optional
    # entry_points={  # Optional
    #     'console_scripts': [
    #         'sample=sample:main',
    #     ],
    # },
    project_urls={  # Optional
        'Funding': 'https://donate.pypi.org',
        'Say Thanks!': 'http://saythanks.io/to/example',
        'Source': 'https://bitbucket.org/maxwin-inc/auth/src/master/',
    },
)


# from setuptools import setup
# from codecs import open
# from os import path
#
# here = path.abspath(path.dirname(__file__))
#
# def readme():
#     with open(path.join(here, 'readme.md'), encoding='utf-8') as f:
#         long_description = f.read()
#     return long_description
# setup(
#     name='mwauth',
#     version='0.4.16',
#     description='maxwin auth ',
#     author='cxhjet',
#     author_email='13064576@qq.com',
#     url='https://bitbucket.org/maxwin-inc/auth/src/master/',
#     long_description=readme(),  # Optional
#     # long_description_content_type='text/markdown',  # Optional (see note above)
#     classifiers=[  # Optional
#         'Development Status :: 5 - Production/Stable',
#         'Intended Audience :: Developers',
#         'Topic :: Software Development :: Build Tools',
#         'License :: OSI Approved :: MIT License',
#         'Programming Language :: Python :: 3.5',
#         'Programming Language :: Python :: 3.6',
#     ],
#     keywords='auth',  # Optional
#     # packages=find_packages(exclude=['test']),  # Required
#     packages=['mwauth','mwauth.static.auth.css',
#               'mwauth.static.auth.fonts','mwauth.static.auth.js',
#               'mwauth.templates.auth'],
#     package_data={'': ['*.*']},
#     # include_package_data=True,
#     install_requires=['flask','flask_redis'],  # Optional
#     # project_urls={  # Optional
#     #     'Bug Reports': 'https://github.com/pypa/sampleproject/issues',
#     #     'Funding': 'https://donate.pypi.org',
#     #     'Say Thanks!': 'http://saythanks.io/to/example',
#     #     'Source': 'https://bitbucket.org/maxwin-inc/auth/src/master/',
#     # },
# )

# # setup(
# #     name=          'mwauth',
# #     version=       '0.4.15',
# #     description=   'maxwin auth ',
# #     author     =   'cxhjet',
# #     author_email = '13064576@qq.com',
# #     url   =        'https://bitbucket.org/maxwin-inc/auth/src/master/',
# #     long_description=long_description,  # Optional
# #     long_description_content_type='text/markdown',  # Optional (see note above)
# #     packages=['mwauth','mwauth.static.auth.css',
# #               'mwauth.static.auth.fonts','mwauth.static.auth.js',
# #               'mwauth.templates.auth'],
# #     package_data={
# #         '': ['*.*']
# #     },
# #     classifiers=[
# #         'Intended Audience :: Developers',
# #         'License :: OSI Approved :: BSD License',
# #         'Operating System :: OS Independent',
# #         'Programming Language :: Python :: 3.5',
# #     ],
# #     # install_requires=[
# #     #     'flask>=0.11.1'
# #     # ],
# # )



__version__ = '1.5.0'
__author__ = 'Flask-Admin team'
__email__ = 'serge.koval+github@gmail.com'

#encoding=utf8
from flask import g
from enum import Enum
from .kong_auth import KongAuth
from .session_auth3 import SessionAuth

class AuthType(Enum):
    '''
    该类型支持程序启动时，选择auth 方式
    '''
    # kong_auth.KongAuth,同时支持jwt和session认证方式
    jwt = 'jwt'
    # session_auth3.SessionAuth，只支持非kong的cookie认证
    session = 'session'

def auth(auth_type=AuthType.jwt.value,app=None):
    '''
    支持在程序启动的时候，选择认证类型
    :param auth_type: 认证类型，比如：jwt，session...
    :param app:
    :return:
    '''
    if auth_type==AuthType.jwt.value:
        return KongAuth(app)
    elif auth_type==AuthType.session.value:
        return SessionAuth(app)
    else:
        assert False,'auth_type(%s) is not supoort!'%auth_type

def current_user():
    return g.user

class BaseUser(object):
    def get_password(self):
        pass
    def check(self,password):
        pass
    def make(self,password):
        pass
    def to_json(self):
        pass
    @staticmethod
    def FindUser(username):
        pass

